@@ -33,6 +33,8 @@ use pyo3::{
     create_exception, exceptions::PyNotImplementedError, pyclass, pyclass::CompareOp, pymethods,
     pymodule, types::PyModule, IntoPy, PyObject, PyResult, Python,
 };
+use schemars::gen::SchemaGenerator;
+use schemars::schema::Schema;
 use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 use thiserror::Error;
 use url::Url;
@@ -153,6 +155,15 @@ pub struct Requirement<T: Pep508Url = VerbatimUrl> {
     pub origin: Option<RequirementOrigin>,
 }
 
+impl<T: Pep508Url> Requirement<T> {
+    /// Removes the URL specifier from this requirement.
+    pub fn clear_url(&mut self) {
+        if matches!(self.version_or_url, Some(VersionOrUrl::Url(_))) {
+            self.version_or_url = None;
+        }
+    }
+}
+
 impl<T: Pep508Url + Display> Display for Requirement<T> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.name)?;
@@ -480,6 +491,25 @@ impl Reporter for TracingReporter {
     }
 }
 
+#[cfg(feature = "schemars")]
+impl<T: Pep508Url> schemars::JsonSchema for Requirement<T> {
+    fn schema_name() -> String {
+        "Requirement".to_string()
+    }
+
+    fn json_schema(_gen: &mut SchemaGenerator) -> Schema {
+        schemars::schema::SchemaObject {
+            instance_type: Some(schemars::schema::InstanceType::String.into()),
+            metadata: Some(Box::new(schemars::schema::Metadata {
+                description: Some("A PEP 508 dependency specifier".to_string()),
+                ..schemars::schema::Metadata::default()
+            })),
+            ..schemars::schema::SchemaObject::default()
+        }
+        .into()
+    }
+}
+
 impl<T: Pep508Url> FromStr for Requirement<T> {
     type Err = Pep508Error<T>;
 
