@@ -63,6 +63,10 @@ static LD_NOT_FOUND_RE: Lazy<Regex> = Lazy::new(|| {
 static WHEEL_NOT_FOUND_RE: Lazy<Regex> =
     Lazy::new(|| Regex::new(r"error: invalid command 'bdist_wheel'").unwrap());
 
+/// e.g. `ModuleNotFoundError: No module named 'torch'`
+static TORCH_NOT_FOUND_RE: Lazy<Regex> =
+    Lazy::new(|| Regex::new(r"ModuleNotFoundError: No module named 'torch'").unwrap());
+
 /// The default backend to use when PEP 517 is used without a `build-system` section.
 static DEFAULT_BACKEND: Lazy<Pep517Backend> = Lazy::new(|| Pep517Backend {
     backend: "setuptools.build_meta:__legacy__".to_string(),
@@ -151,7 +155,9 @@ impl Display for MissingHeaderCause {
             MissingLibrary::PythonPackage(package) => {
                 write!(
                     f,
-                    "This error likely indicates that you need to `uv pip install {package}` into the build environment for {version_id}",
+                    "This error likely indicates that {version_id} depends on {package}, but doesn't declare it as a build dependency. \
+                        If {version_id} is a first-party package, consider adding {package} to its `build-system.requires`. \
+                        Otherwise, `uv pip install {package}` into the environment and re-run with `--no-build-isolation`.",
                     package = package, version_id = self.version_id
                 )
             }
@@ -183,6 +189,8 @@ impl Error {
                 Some(MissingLibrary::Linker(library.to_string()))
             } else if WHEEL_NOT_FOUND_RE.is_match(line.trim()) {
                 Some(MissingLibrary::PythonPackage("wheel".to_string()))
+            } else if TORCH_NOT_FOUND_RE.is_match(line.trim()) {
+                Some(MissingLibrary::PythonPackage("torch".to_string()))
             } else {
                 None
             }
@@ -1239,7 +1247,7 @@ mod test {
         "###);
         insta::assert_snapshot!(
             std::error::Error::source(&err).unwrap(),
-            @"This error likely indicates that you need to `uv pip install wheel` into the build environment for pygraphviz-1.11"
+            @"This error likely indicates that pygraphviz-1.11 depends on wheel, but doesn't declare it as a build dependency. If pygraphviz-1.11 is a first-party package, consider adding wheel to its `build-system.requires`. Otherwise, `uv pip install wheel` into the environment and re-run with `--no-build-isolation`."
         );
     }
 }
