@@ -10,7 +10,7 @@ use tracing::{debug, trace};
 
 use pep508_rs::{RequirementOrigin, VerbatimUrl};
 use pypi_types::{Requirement, RequirementSource};
-use uv_fs::{absolutize_path, Simplified};
+use uv_fs::{absolutize_path, normalize_path, relative_to, Simplified};
 use uv_normalize::PackageName;
 use uv_warnings::warn_user;
 
@@ -25,6 +25,8 @@ pub enum WorkspaceError {
     MissingProject(PathBuf),
     #[error("No workspace found for: `{}`", _0.simplified_display())]
     MissingWorkspace(PathBuf),
+    #[error("The project is marked as unmanaged: `{}`", _0.simplified_display())]
+    NonWorkspace(PathBuf),
     #[error("pyproject.toml section is declared as dynamic, but must be static: `{0}`")]
     DynamicNotAllowed(&'static str),
     #[error("Failed to find directories for glob: `{0}`")]
@@ -44,20 +46,29 @@ pub enum WorkspaceError {
 #[derive(Debug, Clone)]
 #[cfg_attr(test, derive(serde::Serialize))]
 pub struct Workspace {
-    /// The path to the workspace root, the directory containing the top level `pyproject.toml` with
+    /// The path to the workspace root.
+    ///
+    /// The workspace root is the directory containing the top level `pyproject.toml` with
     /// the `uv.tool.workspace`, or the `pyproject.toml` in an implicit single workspace project.
-    root: PathBuf,
+    install_path: PathBuf,
+    /// The same path as `install_path`, but relative to the main workspace.
+    ///
+    /// We use this value to compute relative paths for workspace-to-workspace dependencies. It's an
+    /// empty path for the main workspace.
+    lock_path: PathBuf,
     /// The members of the workspace.
     packages: BTreeMap<PackageName, WorkspaceMember>,
-    /// The sources table from the workspace `pyproject.toml`. It is overridden by the project
-    /// sources.
+    /// The sources table from the workspace `pyproject.toml`.
+    ///
+    /// This table is overridden by the project sources.
     sources: BTreeMap<PackageName, Source>,
 }
 
 impl Workspace {
     /// Find the workspace containing the given path.
     ///
-    /// Unlike the [`ProjectWorkspace`] discovery, this does not require a current project.
+    /// Unlike the [`ProjectWorkspace`] discovery, this does not require a current project. It also
+    /// always uses absolute path, i.e. this method only supports discovering the main workspace.
     ///
     /// Steps of workspace discovery: Start by looking at the closest `pyproject.toml`:
     /// * If it's an explicit workspace root: Collect workspace from this root, we're done.
@@ -69,19 +80,35 @@ impl Workspace {
         path: &Path,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Workspace, WorkspaceError> {
-        let project_root = path
+        let path = absolutize_path(path)
+            .map_err(WorkspaceError::Normalize)?
+            .to_path_buf();
+
+        let project_path = path
             .ancestors()
             .find(|path| path.join("pyproject.toml").is_file())
-            .ok_or(WorkspaceError::MissingPyprojectToml)?;
+            .ok_or(WorkspaceError::MissingPyprojectToml)?
+            .to_path_buf();
 
-        let pyproject_path = project_root.join("pyproject.toml");
+        let pyproject_path = project_path.join("pyproject.toml");
         let contents = fs_err::tokio::read_to_string(&pyproject_path).await?;
         let pyproject_toml = PyProjectToml::from_string(contents)
             .map_err(|err| WorkspaceError::Toml(pyproject_path.clone(), Box::new(err)))?;
 
-        let project_path = absolutize_path(project_root)
-            .map_err(WorkspaceError::Normalize)?
-            .to_path_buf();
+        // Check if the project is explicitly marked as unmanaged.
+        if pyproject_toml
+            .tool
+            .as_ref()
+            .and_then(|tool| tool.uv.as_ref())
+            .and_then(|uv| uv.managed)
+            == Some(false)
+        {
+            debug!(
+                "Project `{}` is marked as unmanaged",
+                project_path.simplified_display()
+            );
+            return Err(WorkspaceError::NonWorkspace(project_path));
+        }
 
         // Check if the current project is also an explicit workspace root.
         let explicit_root = pyproject_toml
@@ -133,6 +160,8 @@ impl Workspace {
                 pyproject_toml,
             });
         Self::collect_members(
+            workspace_root.clone(),
+            // This method supports only absolute paths.
             workspace_root,
             workspace_definition,
             workspace_pyproject_toml,
@@ -181,7 +210,7 @@ impl Workspace {
                         install_path: member.root.clone(),
                         lock_path: member
                             .root
-                            .strip_prefix(&self.root)
+                            .strip_prefix(&self.install_path)
                             .expect("Project must be below workspace root")
                             .to_path_buf(),
                         editable: true,
@@ -198,7 +227,7 @@ impl Workspace {
         let Some(workspace_package) = self
             .packages
             .values()
-            .find(|workspace_package| workspace_package.root() == self.root())
+            .find(|workspace_package| workspace_package.root() == self.install_path())
         else {
             return vec![];
         };
@@ -227,13 +256,19 @@ impl Workspace {
 
     /// The path to the workspace root, the directory containing the top level `pyproject.toml` with
     /// the `uv.tool.workspace`, or the `pyproject.toml` in an implicit single workspace project.
-    pub fn root(&self) -> &PathBuf {
-        &self.root
+    pub fn install_path(&self) -> &PathBuf {
+        &self.install_path
+    }
+
+    /// The same path as `install_path()`, but relative to the main workspace. We use this value
+    /// to compute relative paths for workspace-to-workspace dependencies.
+    pub fn lock_path(&self) -> &PathBuf {
+        &self.lock_path
     }
 
     /// The path to the workspace virtual environment.
     pub fn venv(&self) -> PathBuf {
-        self.root.join(".venv")
+        self.install_path.join(".venv")
     }
 
     /// The members of the workspace.
@@ -249,6 +284,7 @@ impl Workspace {
     /// Collect the workspace member projects from the `members` and `excludes` entries.
     async fn collect_members(
         workspace_root: PathBuf,
+        lock_path: PathBuf,
         workspace_definition: ToolUvWorkspace,
         workspace_pyproject_toml: PyProjectToml,
         current_project: Option<WorkspaceMember>,
@@ -326,6 +362,21 @@ impl Workspace {
                 let pyproject_toml = PyProjectToml::from_string(contents)
                     .map_err(|err| WorkspaceError::Toml(pyproject_path, Box::new(err)))?;
 
+                // Check if the current project is explicitly marked as unmanaged.
+                if pyproject_toml
+                    .tool
+                    .as_ref()
+                    .and_then(|tool| tool.uv.as_ref())
+                    .and_then(|uv| uv.managed)
+                    == Some(false)
+                {
+                    debug!(
+                        "Project `{}` is marked as unmanaged; omitting from workspace members",
+                        pyproject_toml.project.as_ref().unwrap().name
+                    );
+                    continue;
+                }
+
                 // Extract the package name.
                 let Some(project) = pyproject_toml.project.clone() else {
                     return Err(WorkspaceError::MissingProject(member_root));
@@ -354,7 +405,8 @@ impl Workspace {
         check_nested_workspaces(&workspace_root, stop_discovery_at);
 
         Ok(Workspace {
-            root: workspace_root,
+            install_path: workspace_root,
+            lock_path,
             packages: workspace_members,
             sources: workspace_sources,
         })
@@ -509,7 +561,7 @@ impl ProjectWorkspace {
     }
 
     /// Discover the workspace starting from the directory containing the `pyproject.toml`.
-    pub async fn from_project_root(
+    async fn from_project_root(
         project_root: &Path,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Self, WorkspaceError> {
@@ -525,17 +577,25 @@ impl ProjectWorkspace {
             .clone()
             .ok_or_else(|| WorkspaceError::MissingProject(pyproject_path.clone()))?;
 
-        Self::from_project(project_root, &project, &pyproject_toml, stop_discovery_at).await
+        Self::from_project(
+            project_root,
+            Path::new(""),
+            &project,
+            &pyproject_toml,
+            stop_discovery_at,
+        )
+        .await
     }
 
     /// If the current directory contains a `pyproject.toml` with a `project` table, discover the
     /// workspace and return it, otherwise it is a dynamic path dependency and we return `Ok(None)`.
     pub async fn from_maybe_project_root(
-        project_root: &Path,
+        install_path: &Path,
+        lock_path: &Path,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Option<Self>, WorkspaceError> {
         // Read the `pyproject.toml`.
-        let pyproject_path = project_root.join("pyproject.toml");
+        let pyproject_path = install_path.join("pyproject.toml");
         let Ok(contents) = fs_err::tokio::read_to_string(&pyproject_path).await else {
             // No `pyproject.toml`, but there may still be a `setup.py` or `setup.cfg`.
             return Ok(None);
@@ -550,7 +610,14 @@ impl ProjectWorkspace {
         };
 
         Ok(Some(
-            Self::from_project(project_root, &project, &pyproject_toml, stop_discovery_at).await?,
+            Self::from_project(
+                install_path,
+                lock_path,
+                &project,
+                &pyproject_toml,
+                stop_discovery_at,
+            )
+            .await?,
         ))
     }
 
@@ -577,15 +644,28 @@ impl ProjectWorkspace {
 
     /// Find the workspace for a project.
     pub async fn from_project(
-        project_path: &Path,
+        install_path: &Path,
+        lock_path: &Path,
         project: &Project,
         project_pyproject_toml: &PyProjectToml,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Self, WorkspaceError> {
-        let project_path = absolutize_path(project_path)
+        let project_path = absolutize_path(install_path)
             .map_err(WorkspaceError::Normalize)?
             .to_path_buf();
 
+        // Check if workspaces are explicitly disabled for the project.
+        if project_pyproject_toml
+            .tool
+            .as_ref()
+            .and_then(|tool| tool.uv.as_ref())
+            .and_then(|uv| uv.managed)
+            == Some(false)
+        {
+            debug!("Project `{}` is marked as unmanaged", project.name);
+            return Err(WorkspaceError::NonWorkspace(project_path));
+        }
+
         // Check if the current project is also an explicit workspace root.
         let mut workspace = project_pyproject_toml
             .tool
@@ -624,7 +704,9 @@ impl ProjectWorkspace {
                 project_root: project_path.clone(),
                 project_name: project.name.clone(),
                 workspace: Workspace {
-                    root: project_path.clone(),
+                    install_path: project_path.clone(),
+                    // The workspace and the project are the same, so the relative path is, too.
+                    lock_path: lock_path.to_path_buf(),
                     packages: current_project_as_members,
                     // There may be package sources, but we don't need to duplicate them into the
                     // workspace sources.
@@ -638,8 +720,28 @@ impl ProjectWorkspace {
             workspace_root.simplified_display()
         );
 
+        // Say we have:
+        // ```
+        // root
+        // ├── main_workspace  <- The reference point
+        // │   ├── pyproject.toml
+        // │   └── uv.lock
+        // └──current_workspace  <- We want this relative to the main workspace
+        //    └── packages
+        //        └── current_package  <- We have this relative to the main workspace
+        //            └── pyproject.toml
+        // ```
+        // The lock path we need: `../current_workspace`
+        // workspace root: `/root/current_workspace`
+        // project path: `/root/current_workspace/packages/current_project`
+        // relative to workspace: `../..`
+        // lock path: `../current_workspace`
+        let up_to_root = relative_to(&workspace_root, &project_path)?;
+        let lock_path = normalize_path(&lock_path.join(up_to_root));
+
         let workspace = Workspace::collect_members(
             workspace_root,
+            lock_path,
             workspace_definition,
             workspace_pyproject_toml,
             Some(current_project),
@@ -733,7 +835,7 @@ async fn find_workspace(
             // We require that a `project.toml` file either declares a workspace or a project.
             warn_user!(
                 "pyproject.toml does not contain `project` table: `{}`",
-                workspace_root.simplified_display()
+                pyproject_path.simplified_display()
             );
             Ok(None)
         };
@@ -857,10 +959,17 @@ impl VirtualProject {
     ///
     /// Similar to calling [`ProjectWorkspace::discover`] with a fallback to [`Workspace::discover`],
     /// but avoids rereading the `pyproject.toml` (and relying on error-handling as control flow).
+    ///
+    /// This method requires an absolute path and panics otherwise, i.e. this method only supports
+    /// discovering the main workspace.
     pub async fn discover(
         path: &Path,
         stop_discovery_at: Option<&Path>,
     ) -> Result<Self, WorkspaceError> {
+        assert!(
+            path.is_absolute(),
+            "virtual project discovery with relative path"
+        );
         let project_root = path
             .ancestors()
             .take_while(|path| {
@@ -887,6 +996,7 @@ impl VirtualProject {
             // If the `pyproject.toml` contains a `[project]` table, it's a project.
             let project = ProjectWorkspace::from_project(
                 project_root,
+                Path::new(""),
                 project,
                 &pyproject_toml,
                 stop_discovery_at,
@@ -906,6 +1016,7 @@ impl VirtualProject {
 
             let workspace = Workspace::collect_members(
                 project_path,
+                PathBuf::new(),
                 workspace.clone(),
                 pyproject_toml,
                 None,
@@ -987,7 +1098,8 @@ mod tests {
           "project_root": "[ROOT]/albatross-in-example/examples/bird-feeder",
           "project_name": "bird-feeder",
           "workspace": {
-            "root": "[ROOT]/albatross-in-example/examples/bird-feeder",
+            "install_path": "[ROOT]/albatross-in-example/examples/bird-feeder",
+            "lock_path": "",
             "packages": {
               "bird-feeder": {
                 "root": "[ROOT]/albatross-in-example/examples/bird-feeder",
@@ -1022,7 +1134,8 @@ mod tests {
               "project_root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
               "project_name": "bird-feeder",
               "workspace": {
-                "root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
+                "install_path": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
+                "lock_path": "",
                 "packages": {
                   "bird-feeder": {
                     "root": "[ROOT]/albatross-project-in-excluded/excluded/bird-feeder",
@@ -1056,7 +1169,8 @@ mod tests {
               "project_root": "[ROOT]/albatross-root-workspace",
               "project_name": "albatross",
               "workspace": {
-                "root": "[ROOT]/albatross-root-workspace",
+                "install_path": "[ROOT]/albatross-root-workspace",
+                "lock_path": "",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-root-workspace",
@@ -1114,7 +1228,8 @@ mod tests {
               "project_root": "[ROOT]/albatross-virtual-workspace/packages/albatross",
               "project_name": "albatross",
               "workspace": {
-                "root": "[ROOT]/albatross-virtual-workspace",
+                "install_path": "[ROOT]/albatross-virtual-workspace",
+                "lock_path": "../..",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-virtual-workspace/packages/albatross",
@@ -1166,7 +1281,8 @@ mod tests {
               "project_root": "[ROOT]/albatross-just-project",
               "project_name": "albatross",
               "workspace": {
-                "root": "[ROOT]/albatross-just-project",
+                "install_path": "[ROOT]/albatross-just-project",
+                "lock_path": "",
                 "packages": {
                   "albatross": {
                     "root": "[ROOT]/albatross-just-project",
