@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::io;
 use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus};
@@ -123,21 +124,21 @@ impl Interpreter {
     }
 
     /// Return a new [`Interpreter`] to install into the given `--target` directory.
-    #[must_use]
-    pub fn with_target(self, target: Target) -> Self {
-        Self {
+    pub fn with_target(self, target: Target) -> io::Result<Self> {
+        target.init()?;
+        Ok(Self {
             target: Some(target),
             ..self
-        }
+        })
     }
 
     /// Return a new [`Interpreter`] to install into the given `--prefix` directory.
-    #[must_use]
-    pub fn with_prefix(self, prefix: Prefix) -> Self {
-        Self {
+    pub fn with_prefix(self, prefix: Prefix) -> io::Result<Self> {
+        prefix.init(self.virtualenv())?;
+        Ok(Self {
             prefix: Some(prefix),
             ..self
-        }
+        })
     }
 
     /// Return the [`Interpreter`] for the base executable, if it's available.
@@ -430,17 +431,40 @@ impl Interpreter {
         }
     }
 
-    /// Return an iterator over the `site-packages` directories inside the environment.
-    pub fn site_packages(&self) -> impl Iterator<Item = &Path> {
-        let purelib = self.purelib();
-        let platlib = self.platlib();
-        std::iter::once(purelib).chain(
-            if purelib == platlib || is_same_file(purelib, platlib).unwrap_or(false) {
-                None
-            } else {
-                Some(platlib)
-            },
-        )
+    /// Returns an iterator over the `site-packages` directories inside the environment.
+    ///
+    /// In most cases, `purelib` and `platlib` will be the same, and so the iterator will contain
+    /// a single element; however, in some distributions, they may be different.
+    ///
+    /// Some distributions also create symbolic links from `purelib` to `platlib`; in such cases, we
+    /// still deduplicate the entries, returning a single path.
+    pub fn site_packages(&self) -> impl Iterator<Item = Cow<Path>> {
+        let target = self.target().map(Target::site_packages);
+
+        let prefix = self
+            .prefix()
+            .map(|prefix| prefix.site_packages(self.virtualenv()));
+
+        let interpreter = if target.is_none() && prefix.is_none() {
+            let purelib = self.purelib();
+            let platlib = self.platlib();
+            Some(std::iter::once(purelib).chain(
+                if purelib == platlib || is_same_file(purelib, platlib).unwrap_or(false) {
+                    None
+                } else {
+                    Some(platlib)
+                },
+            ))
+        } else {
+            None
+        };
+
+        target
+            .into_iter()
+            .flatten()
+            .map(Cow::Borrowed)
+            .chain(prefix.into_iter().flatten().map(Cow::Owned))
+            .chain(interpreter.into_iter().flatten().map(Cow::Borrowed))
     }
 
     /// Check if the interpreter matches the given Python version.
