@@ -255,7 +255,8 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
 
                 SourceUrl::Directory(DirectorySourceUrl {
                     url: &requirement.url.verbatim,
-                    path: Cow::Borrowed(&parsed_directory_url.install_path),
+                    install_path: Cow::Borrowed(&parsed_directory_url.install_path),
+                    lock_path: Cow::Borrowed(&parsed_directory_url.lock_path),
                     editable: parsed_directory_url.editable,
                 })
             }
