@@ -155,6 +155,11 @@ impl RequiresPython {
         self.bound.as_ref() == Bound::Unbounded
     }
 
+    /// Returns the [`RequiresPythonBound`] for the `Requires-Python` specifier.
+    pub fn bound(&self) -> &RequiresPythonBound {
+        &self.bound
+    }
+
     /// Returns this `Requires-Python` specifier as an equivalent marker
     /// expression utilizing the `python_version` marker field.
     ///
@@ -254,6 +259,16 @@ impl RequiresPythonBound {
     }
 }
 
+impl From<RequiresPythonBound> for Range<Version> {
+    fn from(value: RequiresPythonBound) -> Self {
+        match value.0 {
+            Bound::Included(version) => Range::higher_than(version),
+            Bound::Excluded(version) => Range::strictly_higher_than(version),
+            Bound::Unbounded => Range::full(),
+        }
+    }
+}
+
 impl Deref for RequiresPythonBound {
     type Target = Bound<Version>;
 
