@@ -7,44 +7,15 @@ use assert_cmd::prelude::*;
 use assert_fs::fixture::ChildPath;
 use assert_fs::prelude::*;
 
-use crate::common::{get_bin, uv_snapshot, TestContext, EXCLUDE_NEWER};
+use crate::common::{get_bin, uv_snapshot, TestContext};
 
 mod common;
 
 /// Create a `pip freeze` command with options shared across scenarios.
 fn command(context: &TestContext) -> Command {
     let mut command = Command::new(get_bin());
-    command
-        .arg("pip")
-        .arg("freeze")
-        .arg("--cache-dir")
-        .arg(context.cache_dir.path())
-        .env("VIRTUAL_ENV", context.venv.as_os_str())
-        .env("UV_NO_WRAP", "1")
-        .current_dir(&context.temp_dir);
-    command
-}
-
-/// Create a `pip install` command with options shared across scenarios.
-fn sync_command(context: &TestContext) -> Command {
-    let mut command = Command::new(get_bin());
-    command
-        .arg("pip")
-        .arg("sync")
-        .arg("--cache-dir")
-        .arg(context.cache_dir.path())
-        .arg("--exclude-newer")
-        .arg(EXCLUDE_NEWER)
-        .env("VIRTUAL_ENV", context.venv.as_os_str())
-        .env("UV_NO_WRAP", "1")
-        .current_dir(&context.temp_dir);
-
-    if cfg!(all(windows, debug_assertions)) {
-        // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
-        // default windows stack of 1MB
-        command.env("UV_STACK_SIZE", (2 * 1024 * 1024).to_string());
-    }
-
+    command.arg("pip").arg("freeze");
+    context.add_shared_args(&mut command);
     command
 }
 
@@ -57,7 +28,8 @@ fn freeze_many() -> Result<()> {
     requirements_txt.write_str("MarkupSafe==2.1.3\ntomli==2.0.1")?;
 
     // Run `pip sync`.
-    sync_command(&context)
+    context
+        .pip_sync()
         .arg(requirements_txt.path())
         .assert()
         .success();
@@ -90,7 +62,8 @@ fn freeze_duplicate() -> Result<()> {
     requirements_txt.write_str("pip==21.3.1")?;
 
     // Run `pip sync`.
-    sync_command(&context1)
+    context1
+        .pip_sync()
         .arg(requirements_txt.path())
         .assert()
         .success();
@@ -101,7 +74,8 @@ fn freeze_duplicate() -> Result<()> {
     requirements_txt.write_str("pip==22.1.1")?;
 
     // Run `pip sync`.
-    sync_command(&context2)
+    context2
+        .pip_sync()
         .arg(requirements_txt.path())
         .assert()
         .success();
@@ -139,7 +113,8 @@ fn freeze_url() -> Result<()> {
     requirements_txt.write_str("anyio\niniconfig @ https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl")?;
 
     // Run `pip sync`.
-    sync_command(&context)
+    context
+        .pip_sync()
         .arg(requirements_txt.path())
         .assert()
         .success();
@@ -176,7 +151,8 @@ fn freeze_with_editable() -> Result<()> {
     ))?;
 
     // Run `pip sync`.
-    sync_command(&context)
+    context
+        .pip_sync()
         .arg(requirements_txt.path())
         .assert()
         .success();
@@ -221,7 +197,7 @@ fn freeze_with_egg_info() -> Result<()> {
 
     let site_packages = ChildPath::new(context.site_packages());
 
-    // Manually create a `.egg-info` directory.
+    // Manually create an `.egg-info` directory.
     site_packages
         .child("zstandard-0.22.0-py3.12.egg-info")
         .create_dir_all()?;
@@ -266,6 +242,96 @@ fn freeze_with_egg_info() -> Result<()> {
     Ok(())
 }
 
+/// Show an `.egg-info` package in a virtual environment. In this case, the filename omits the
+/// Python version.
+#[test]
+fn freeze_with_egg_info_no_py() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    let site_packages = ChildPath::new(context.site_packages());
+
+    // Manually create an `.egg-info` directory.
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .create_dir_all()?;
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .child("top_level.txt")
+        .write_str("zstd")?;
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .child("SOURCES.txt")
+        .write_str("")?;
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .child("PKG-INFO")
+        .write_str("")?;
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .child("dependency_links.txt")
+        .write_str("")?;
+    site_packages
+        .child("zstandard-0.22.0.egg-info")
+        .child("entry_points.txt")
+        .write_str("")?;
+
+    // Manually create the package directory.
+    site_packages.child("zstd").create_dir_all()?;
+    site_packages
+        .child("zstd")
+        .child("__init__.py")
+        .write_str("")?;
+
+    // Run `pip freeze`.
+    uv_snapshot!(context.filters(), command(&context), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    zstandard==0.22.0
+
+    ----- stderr -----
+    "###);
+
+    Ok(())
+}
+
+/// Show a set of `.egg-info` files in a virtual environment.
+#[test]
+fn freeze_with_egg_info_file() -> Result<()> {
+    let context = TestContext::new("3.11");
+    let site_packages = ChildPath::new(context.site_packages());
+
+    // Manually create a `.egg-info` file with python version.
+    site_packages
+        .child("pycurl-7.45.1-py3.11.egg-info")
+        .write_str(indoc::indoc! {"
+            Metadata-Version: 1.1
+            Name: pycurl
+            Version: 7.45.1
+        "})?;
+
+    // Manually create another `.egg-info` file with no python version.
+    site_packages
+        .child("vtk-9.2.6.egg-info")
+        .write_str(indoc::indoc! {"
+            Metadata-Version: 1.1
+            Name: vtk
+            Version: 9.2.6
+        "})?;
+
+    // Run `pip freeze`.
+    uv_snapshot!(context.filters(), command(&context), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    pycurl==7.45.1
+    vtk==9.2.6
+
+    ----- stderr -----
+    "###);
+    Ok(())
+}
+
 #[test]
 fn freeze_with_legacy_editable() -> Result<()> {
     let context = TestContext::new("3.12");
