@@ -26,7 +26,8 @@ fn tool_install() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -42,7 +43,7 @@ fn tool_install() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -101,7 +102,8 @@ fn tool_install() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("flask")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -118,7 +120,7 @@ fn tool_install() {
      + jinja2==3.1.3
      + markupsafe==2.1.5
      + werkzeug==3.0.1
-    Installed: flask
+    Installed 1 executable: flask
     "###);
 
     tool_dir.child("flask").assert(predicate::path::is_dir());
@@ -168,6 +170,69 @@ fn tool_install() {
     });
 }
 
+#[test]
+fn tool_install_suggest_other_packages_with_executable() {
+    let context = TestContext::new("3.12").with_filtered_exe_suffix();
+    let tool_dir = context.temp_dir.child("tools");
+    let bin_dir = context.temp_dir.child("bin");
+    let mut filters = context.filters();
+    filters.push(("\\+ uvloop(.+)\n ", ""));
+
+    uv_snapshot!(filters, context.tool_install_without_exclude_newer()
+    .arg("fastapi==0.111.0")
+    .env("UV_EXCLUDE_NEWER", "2024-05-04T00:00:00Z") // TODO: Remove this once EXCLUDE_NEWER is bumped past 2024-05-04
+    // (FastAPI 0.111 is only available from this date onwards)
+    .env("UV_TOOL_DIR", tool_dir.as_os_str())
+    .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    No executables are provided by package `fastapi`.
+    However, an executable with the name `fastapi` is available via dependency `fastapi-cli`.
+    Did you mean `uv tool install fastapi-cli`?
+
+    ----- stderr -----
+    warning: `uv tool install` is experimental and may change without warning.
+    Resolved 35 packages in [TIME]
+    Prepared 35 packages in [TIME]
+    Installed 35 packages in [TIME]
+     + annotated-types==0.6.0
+     + anyio==4.3.0
+     + certifi==2024.2.2
+     + click==8.1.7
+     + dnspython==2.6.1
+     + email-validator==2.1.1
+     + fastapi==0.111.0
+     + fastapi-cli==0.0.2
+     + h11==0.14.0
+     + httpcore==1.0.5
+     + httptools==0.6.1
+     + httpx==0.27.0
+     + idna==3.7
+     + jinja2==3.1.3
+     + markdown-it-py==3.0.0
+     + markupsafe==2.1.5
+     + mdurl==0.1.2
+     + orjson==3.10.3
+     + pydantic==2.7.1
+     + pydantic-core==2.18.2
+     + pygments==2.17.2
+     + python-dotenv==1.0.1
+     + python-multipart==0.0.9
+     + pyyaml==6.0.1
+     + rich==13.7.1
+     + shellingham==1.5.4
+     + sniffio==1.3.1
+     + starlette==0.37.2
+     + typer==0.12.3
+     + typing-extensions==4.11.0
+     + ujson==5.9.0
+     + uvicorn==0.29.0
+     + watchfiles==0.21.0
+     + websockets==12.0
+    "###);
+}
+
 /// Test installing a tool at a version
 #[test]
 fn tool_install_version() {
@@ -179,7 +244,8 @@ fn tool_install_version() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black==24.2.0")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -195,7 +261,7 @@ fn tool_install_version() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -264,7 +330,8 @@ fn tool_install_from() {
         .arg("--from")
         .arg("black==24.2.0")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -280,7 +347,7 @@ fn tool_install_from() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     // Attempt to install `black` using `--from` with a different package name
@@ -289,14 +356,15 @@ fn tool_install_from() {
         .arg("--from")
         .arg("flask==24.2.0")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    error: Package name `flask` provided with `--from` does not match install request `black`
+    error: Package name (`flask`) provided with `--from` does not match install request (`black`)
     "###);
 
     // Attempt to install `black` using `--from` with a different version
@@ -305,14 +373,15 @@ fn tool_install_from() {
         .arg("--from")
         .arg("black==24.3.0")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    error: Package requirement `black==24.3.0` provided with `--from` conflicts with install request `black==24.2.0`
+    error: Package requirement (`black==24.3.0`) provided with `--from` conflicts with install request (`black==24.2.0`)
     "###);
 }
 
@@ -329,7 +398,8 @@ fn tool_install_already_installed() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -345,7 +415,7 @@ fn tool_install_already_installed() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -394,14 +464,15 @@ fn tool_install_already_installed() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
-    success: false
-    exit_code: 1
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
+    success: true
+    exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Tool `black` is already installed
+    `black` is already installed
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -429,7 +500,8 @@ fn tool_install_already_installed() {
         .arg("black")
         .arg("--reinstall")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -451,7 +523,7 @@ fn tool_install_already_installed() {
      + pathspec==0.12.1
      - platformdirs==4.2.0
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     // Install `black` again with `--reinstall-package` for `black`
@@ -461,7 +533,8 @@ fn tool_install_already_installed() {
         .arg("--reinstall-package")
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -473,7 +546,7 @@ fn tool_install_already_installed() {
     Installed [N] packages in [TIME]
      - black==24.3.0
      + black==24.3.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     // Install `black` again with `--reinstall-package` for a dependency
@@ -483,7 +556,8 @@ fn tool_install_already_installed() {
         .arg("--reinstall-package")
         .arg("click")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -495,7 +569,7 @@ fn tool_install_already_installed() {
     Installed [N] packages in [TIME]
      - click==8.1.7
      + click==8.1.7
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 }
 
@@ -515,7 +589,8 @@ fn tool_install_entry_point_exists() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -531,7 +606,7 @@ fn tool_install_entry_point_exists() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    error: Entry point for tool already exists: black (use `--force` to overwrite)
+    error: Executable already exists: black (use `--force` to overwrite)
     "###);
 
     // We should delete the virtual environment
@@ -554,7 +629,8 @@ fn tool_install_entry_point_exists() {
         .arg("black")
         .arg("--reinstall")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -569,7 +645,7 @@ fn tool_install_entry_point_exists() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    error: Entry point for tool already exists: black (use `--force` to overwrite)
+    error: Executable already exists: black (use `--force` to overwrite)
     "###);
 
     // We should not create a virtual environment
@@ -594,7 +670,8 @@ fn tool_install_entry_point_exists() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -609,7 +686,7 @@ fn tool_install_entry_point_exists() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    error: Entry points for tool already exist: black, blackd (use `--force` to overwrite)
+    error: Executables already exist: black, blackd (use `--force` to overwrite)
     "###);
 
     // Install `black` with `--force`
@@ -617,7 +694,8 @@ fn tool_install_entry_point_exists() {
         .arg("black")
         .arg("--force")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -632,7 +710,7 @@ fn tool_install_entry_point_exists() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -642,14 +720,15 @@ fn tool_install_entry_point_exists() {
         .arg("black")
         .arg("--force")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -658,14 +737,15 @@ fn tool_install_entry_point_exists() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
-    success: false
-    exit_code: 1
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
+    success: true
+    exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Tool `black` is already installed
+    `black` is already installed
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -675,7 +755,8 @@ fn tool_install_entry_point_exists() {
         .arg("black")
         .arg("--reinstall")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -697,7 +778,7 @@ fn tool_install_entry_point_exists() {
      + pathspec==0.12.1
      - platformdirs==4.2.0
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -770,7 +851,18 @@ fn tool_install_home() {
     let tool_dir = context.temp_dir.child("tools");
 
     // Install `black`
-    uv_snapshot!(context.filters(), context.tool_install().arg("black").env("UV_TOOL_DIR", tool_dir.as_os_str()), @r###"
+    let mut cmd = context.tool_install();
+    cmd.arg("black")
+        .env("UV_TOOL_DIR", tool_dir.as_os_str())
+        .env(
+            "XDG_DATA_HOME",
+            context.home_dir.child(".local").child("share").as_os_str(),
+        )
+        .env(
+            "PATH",
+            context.home_dir.child(".local").child("bin").as_os_str(),
+        );
+    uv_snapshot!(context.filters(), cmd, @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -786,7 +878,7 @@ fn tool_install_home() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     context
@@ -801,12 +893,14 @@ fn tool_install_xdg_data_home() {
     let context = TestContext::new("3.12").with_filtered_exe_suffix();
     let tool_dir = context.temp_dir.child("tools");
     let data_home = context.temp_dir.child("data/home");
+    let bin_dir = context.temp_dir.child("data/bin");
 
     // Install `black`
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_DATA_HOME", data_home.as_os_str()), @r###"
+        .env("XDG_DATA_HOME", data_home.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -822,7 +916,7 @@ fn tool_install_xdg_data_home() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     context
@@ -842,7 +936,8 @@ fn tool_install_xdg_bin_home() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -858,7 +953,7 @@ fn tool_install_xdg_bin_home() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     bin_dir
@@ -876,10 +971,12 @@ fn tool_install_no_entrypoints() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("iniconfig")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
-    exit_code: 2
+    exit_code: 1
     ----- stdout -----
+    No executables are provided by package `iniconfig`.
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
@@ -887,7 +984,6 @@ fn tool_install_no_entrypoints() {
     Prepared 1 package in [TIME]
     Installed 1 package in [TIME]
      + iniconfig==2.0.0
-    error: No entry points found for tool `iniconfig`
     "###);
 }
 
@@ -902,7 +998,8 @@ fn tool_install_unnamed_package() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("https://files.pythonhosted.org/packages/0f/89/294c9a6b6c75a08da55e9d05321d0707e9418735e3062b12ef0f54c33474/black-24.4.2-py3-none-any.whl")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -918,7 +1015,7 @@ fn tool_install_unnamed_package() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -988,14 +1085,15 @@ fn tool_install_unnamed_conflict() {
         .arg("--from")
         .arg("https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    error: Package name `iniconfig` provided with `--from` does not match install request `black`
+    error: Package name (`iniconfig`) provided with `--from` does not match install request (`black`)
     "###);
 }
 
@@ -1012,7 +1110,8 @@ fn tool_install_unnamed_from() {
         .arg("--from")
         .arg("https://files.pythonhosted.org/packages/0f/89/294c9a6b6c75a08da55e9d05321d0707e9418735e3062b12ef0f54c33474/black-24.4.2-py3-none-any.whl")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1028,7 +1127,7 @@ fn tool_install_unnamed_from() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -1097,7 +1196,8 @@ fn tool_install_unnamed_with() {
         .arg("--with")
         .arg("https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1114,7 +1214,7 @@ fn tool_install_unnamed_with() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     tool_dir.child("black").assert(predicate::path::is_dir());
@@ -1186,7 +1286,8 @@ fn tool_install_upgrade() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black==24.1.1")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1202,7 +1303,7 @@ fn tool_install_upgrade() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     insta::with_settings!({
@@ -1224,14 +1325,15 @@ fn tool_install_upgrade() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     insta::with_settings!({
@@ -1254,7 +1356,8 @@ fn tool_install_upgrade() {
         .arg("--with")
         .arg("iniconfig @ https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1265,7 +1368,7 @@ fn tool_install_upgrade() {
     Prepared [N] packages in [TIME]
     Installed [N] packages in [TIME]
      + iniconfig==2.0.0 (from https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl)
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     insta::with_settings!({
@@ -1291,7 +1394,8 @@ fn tool_install_upgrade() {
         .arg("black")
         .arg("--upgrade")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1305,7 +1409,7 @@ fn tool_install_upgrade() {
      - black==24.1.1
      + black==24.3.0
      - iniconfig==2.0.0 (from https://files.pythonhosted.org/packages/ef/a6/62565a6e1cf69e10f5727360368e451d4b7f58beeac6173dc9db836a5b46/iniconfig-2.0.0-py3-none-any.whl)
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     insta::with_settings!({
@@ -1338,7 +1442,8 @@ fn tool_install_python_request() {
         .arg("3.12")
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1354,7 +1459,7 @@ fn tool_install_python_request() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     // Install with Python 3.12 (compatible).
@@ -1363,14 +1468,15 @@ fn tool_install_python_request() {
         .arg("3.12")
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
-    success: false
-    exit_code: 1
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
+    success: true
+    exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Tool `black` is already installed
+    `black` is already installed
     "###);
 
     // Install with Python 3.11 (incompatible).
@@ -1379,14 +1485,15 @@ fn tool_install_python_request() {
         .arg("3.11")
         .arg("black")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Existing environment for `black` does not satisfy the requested Python interpreter: `Python 3.11`
+    Existing environment for `black` does not satisfy the requested Python interpreter
     Resolved [N] packages in [TIME]
     Prepared [N] packages in [TIME]
     Installed [N] packages in [TIME]
@@ -1396,7 +1503,7 @@ fn tool_install_python_request() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 }
 
@@ -1413,7 +1520,8 @@ fn tool_install_preserve_environment() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black==24.1.1")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -1429,7 +1537,7 @@ fn tool_install_preserve_environment() {
      + packaging==24.0
      + pathspec==0.12.1
      + platformdirs==4.2.0
-    Installed: black, blackd
+    Installed 2 executables: black, blackd
     "###);
 
     // Install `black`, but with an incompatible requirement.
@@ -1438,7 +1546,8 @@ fn tool_install_preserve_environment() {
         .arg("--with")
         .arg("packaging==0.0.1")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -1453,13 +1562,50 @@ fn tool_install_preserve_environment() {
     uv_snapshot!(context.filters(), context.tool_install()
         .arg("black==24.1.1")
         .env("UV_TOOL_DIR", tool_dir.as_os_str())
-        .env("XDG_BIN_HOME", bin_dir.as_os_str()), @r###"
-    success: false
-    exit_code: 1
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env("PATH", bin_dir.as_os_str()), @r###"
+    success: true
+    exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
     warning: `uv tool install` is experimental and may change without warning.
-    Tool `black==24.1.1` is already installed
+    `black==24.1.1` is already installed
+    "###);
+}
+
+/// Test warning when the binary directory is not on the user's PATH.
+#[test]
+#[cfg(unix)]
+fn tool_install_warn_path() {
+    let context = TestContext::new("3.12")
+        .with_filtered_counts()
+        .with_filtered_exe_suffix();
+    let tool_dir = context.temp_dir.child("tools");
+    let bin_dir = context.temp_dir.child("bin");
+
+    // Install `black`.
+    uv_snapshot!(context.filters(), context.tool_install()
+        .arg("black==24.1.1")
+        .env("UV_TOOL_DIR", tool_dir.as_os_str())
+        .env("XDG_BIN_HOME", bin_dir.as_os_str())
+        .env_remove("PATH"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv tool install` is experimental and may change without warning.
+    Resolved [N] packages in [TIME]
+    Prepared [N] packages in [TIME]
+    Installed [N] packages in [TIME]
+     + black==24.1.1
+     + click==8.1.7
+     + mypy-extensions==1.0.0
+     + packaging==24.0
+     + pathspec==0.12.1
+     + platformdirs==4.2.0
+    Installed 2 executables: black, blackd
+    warning: [TEMP_DIR]/bin is not on your PATH. To use installed tools, run export PATH="[TEMP_DIR]/bin:$PATH" or uv tool update-shell.
     "###);
 }
